// lav2maf.c -- convert a blastz output file to a .maf file
#include "util.h"
#include "maf.h"
#include "mz_scores.h"
#include "seq.h"

#define MAX_ALIGN_LEN 1000000

char t1[MAX_ALIGN_LEN], t2[MAX_ALIGN_LEN];

int main(int argc, char **argv) {
	FILE *fp;
	SEQ *sf1, *sf2;
	unsigned char *s1, *s2;
	char buf[500];
	int i, j, k, b1, b2, e1, e2, old_e1, old_e2, len1, len2;
	struct mafAli A, *a;
	struct mafComp C1, C2, *c1, *c2;

	argv0 = "lav2maf";
	if (argc != 4)
		fatal("args: blastz.out seqfile1 seqfile2");
	
	sf1 = seq_get(argv[2]);
	s1 = SEQ_CHARS(sf1) - 1;
	len1 = SEQ_LEN(sf1);
	sf2 = seq_get(argv[3]);
	s2 = SEQ_CHARS(sf2) - 1;
	len2 = SEQ_LEN(sf2);

	mafWriteStart(stdout, "lav2maf");
	
	a = &A;
	c1 = &C1;
	c2 = &C2;
	a->components = c1;
	a->score = 0.0;
	c1->next = c2;
	c2->next = NULL;
	c1->src = argv[2];
	c1->srcSize = SEQ_LEN(sf1);
	c1->strand = '+';
	c1->text = t1;
	c2->src = argv[3];
	c2->srcSize = SEQ_LEN(sf2);
	c2->strand = '+';
	c2->text = t2;
	init_scores70();

	fp = ckopen(argv[1], "r");
	if (fgets(buf, 500, fp) == NULL || strcmp(buf, "#:lav\n"))
		fatalf("%s is not a blastz.out file", argv[1]);
	while (fgets(buf, 500, fp) && strcmp(buf, "#:lav\n"))
		;
	while (fgets(buf, 500, fp))
		if (same_string(buf, "#:lav\n")) {
			if (c1->strand == '-')
				fatal("cannot handle multi-contig sequences");
			c2->strand = '-';
		} else if (same_string(buf, "a {\n")) {
			fgets(buf, 500, fp);	// score
			fgets(buf, 500, fp); // can't trust this b1, b2;
			fgets(buf, 500, fp); // ditto for e1, e2
			if (sscanf(buf, "  e %d %d", &e1, &e2) != 2)
				fatalf("cannot parse: %s", buf);
			if (e1 > len1)
				fatal("first sequence name is incorrect");
			if (e2 > len2)
				fatal("second sequence name is incorrect");
			old_e1 = old_e2 = - 1;
			k = 0;
			while (fgets(buf, 500, fp) && buf[0] != '}') {
				if (sscanf(buf, "  l %d %d %d %d",
				  &b1, &b2, &e1, &e2) != 4)
					fatalf("cannot read end-points: %s",
					  buf);
				if (old_e1 == -1) {
					c1->start = b1 - 1;
					c2->start = b2 - 1;
				} else {
					if (k + MAX(e1-c1->start,e2-c2->start)
					   >= MAX_ALIGN_LEN)
					   fatal("laf2maf: alignment too long");
					for (j = old_e1 + 1; j < b1; ++j, ++k) {
						t1[k] = s1[j];
						t2[k] = '-';
					}
					for (j = old_e2 + 1; j < b2; ++j, ++k) {
						t1[k] = '-';
						t2[k] = s2[j];
					}
				}
				for (i = b1, j = b2; i <= e1; ++i, ++j, ++k) {
					t1[k] = s1[i];
					t2[k] = s2[j];
				}
				old_e1 = e1;
				old_e2 = e2;
			}
			c1->size = e1 - c1->start;
			c2->size = e2 - c2->start;
			t1[k] = t2[k] = '\0';
			a->textSize = k;
			a->score = mafScoreRange(a, 0, k);
			mafWrite(stdout, a);
		}

	return 0;
}
