// mz_util.h 

// create a new struct mafComp and initialize some of it
struct mafComp *mafNewComp(char *src, int start, int size, char strand,
  int srcSize, int len);

// locate column of struct mafAli with a given position in sequence 1
int mafPos2Col(struct mafComp *c, int pos, int textSize);

// form a maf from a run of consecutive columns of a given maf
struct mafAli *mafSlice(struct mafAli *a, int start_col, int beyond_col);

/* ------------ start of code to manage the output list of alignments ----------
*  The output alignments are not guarateed to be found in increasing order of start
*  position in the reference sequence.  As they are generated, we buffer them into a
*  properly sorted list, and frequently flush the list.
*/

// flush output-list entries that start before a specified position
struct mafAli *mafFlush(struct mafAli *output_list, int pos);

// insert an alignment into the output list, sorting by components->start
struct mafAli *mafInsert(struct mafAli *output_list, struct mafAli *a);

/* --------- start of code to detect unused segments of alignments -------------
*  We want to print portions of the alignments from the input files that aren't
*  put into an alignment with more rows.  Each input alignment is augumented
*  in a "struct node" with an array with one entry per column of the alignment.
*  Whenever a segment of that alignment is successfully made part of an
*  alignment with more rows, the corresponding columns are marked as "used".
*  When the left-to-right sweep along the reference sequence has completed its
*  use of an input node (remember: we allow alignments in a file to overlap in
*  the reference sequence), we can identify the unused segments.
*/

struct node {
	struct mafAli *maf;
	char *used;
	struct node *next;
};

struct node *nodeNew(struct mafAli *maf);

void nodeFree(struct node **n);

// Mark node's used array according to positions in maf's first component.
void nodeMark(struct node *n, int beg, int end);

/* ----------- end of code to detect unused segments of alignments ---------- */

